@@ -131,6 +131,9 @@ impl Pool {
     }
 
     /// Spawns a closure into a process of the pool.
+    ///
+    /// This works exactly like [`procspawn::spawn`](fn.spawn.html) but instead
+    /// of spawning a new process, it reuses a process from the pool.
     pub fn spawn<
         F: FnOnce(A) -> R + Copy,
         A: Serialize + for<'de> Deserialize<'de>,
@@ -140,6 +143,7 @@ impl Pool {
         args: A,
         func: F,
     ) -> JoinHandle<R> {
+        assert_empty_closure!(F);
         self.assert_alive();
         let _func = func;
         let (args_tx, args_rx) = ipc::channel().unwrap();
